@@ -1,11 +1,20 @@
-__author__ = 'Quentin Kaiser'
-__license__ = "Apache 2.0"
-__version__ = "0.1"
-__contact__ = "kaiserquentin@gmail.com"
-__date__ = "2014/16/11"
+"""
+Copyright 2014 Quentin Kaiser
 
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
 
-class Tag(object):
+You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+"""
+from nessusobject import NessusObject
+
+class Tag(NessusObject):
     """
     A Nessus Tag instance.
 
@@ -15,23 +24,15 @@ class Tag(object):
     http://google-styleguide.googlecode.com/svn/trunk/pyguide.html
     """
 
-    def __init__(self):
+    def __init__(self, server):
         """Constructor"""
-        self._id = -1
+        super(Tag, self).__init__(server)
         self._default_tag = 1
         self._type = None
         self._name = None
         self._custom = False
         self._unread_count = 0
 
-    @property
-    def id(self):
-        return self._id
-
-    @id.setter
-    def id(self, _id):
-        self._id = _id
-
     @property
     def default_tag(self):
         return self._default_tag
