@@ -1,23 +1,31 @@
-__author__ = 'Quentin Kaiser'
-__license__ = "Apache 2.0"
-__version__ = "0.1"
-__contact__ = "kaiserquentin@gmail.com"
-__date__ = "2014/16/11"
+"""
+Copyright 2014 Quentin Kaiser
 
-from httplib import HTTPSConnection, CannotSendRequest, ImproperConnectionState
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+
+You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+"""
+
+from httplib import HTTPSConnection, CannotSendRequest, ImproperConnectionState, HTTPException
 from random import randint
 import os
 import json
 from xml.dom.minidom import parseString
 from time import sleep
 
-from models.scan import Scan
+from models.scan import Scan, Vulnerability, Note, Host, Remediation
+from models.schedule import Schedule
 from models.tag import Tag
-from models.policy import Policy
+from models.policy import Policy, Plugin, PluginFamily, PreferenceValue, Preference
 from models.user import User
 from models.report import Report
-from models.plugin import PluginFamily, Plugin
-from models.preference import PreferenceValue, Preference
 
 
 class NessusAPIError(Exception):
@@ -78,6 +86,27 @@ class Nessus(object):
 
         self._headers = {"Content-type": "application/json", "Accept": "application/json"}
 
+    def Scan(self):
+        return Scan(self)
+
+    def Host(self):
+        return Host(self)
+
+    def Policy(self):
+        return Policy(self)
+
+    def Schedule(self):
+        return Schedule(self)
+
+    def User(self, username=None, password=None):
+        return User(self, username, password)
+
+    def Tag(self):
+        return Tag(self)
+
+    def Report(self):
+        return Report(self)
+
     def _request(self, method, target, params, headers=None):
         """
         Send an HTTP request.
@@ -246,7 +275,7 @@ class Nessus(object):
             if "result" in response:
                 self._scans = []
                 for result in response['result']:
-                    scan = Scan()
+                    scan = self.Scan()
                     scan.status = result["status"]
                     scan.name = result["name"]
                     scan.tag = result["tags"]
@@ -280,7 +309,7 @@ class Nessus(object):
         if response is not None:
             self._tags = []
             for result in response['tags']:
-                tag = Tag()
+                tag = self.Tag()
                 tag.id = result["id"]
                 tag.type = result["type"] if "type" in result else "local"
                 tag.custom = result["custom"]
@@ -304,7 +333,7 @@ class Nessus(object):
             if "policy" in response["policies"]:
                 self._policies = []
                 for result in response['policies']["policy"]:
-                    policy = Policy()
+                    policy = self.Policy()
                     policy.id = result["id"]
                     policy.db_id = result["db_id"]
                     policy.name = result["name"]
@@ -334,7 +363,7 @@ class Nessus(object):
         if response is not None:
             self._users = []
             for result in response["user"]:
-                user = User()
+                user = self.User()
                 user.last_login = result["lastlogin"]
                 user.permissions = result["permissions"]
                 user.type = result["type"]
@@ -358,9 +387,13 @@ class Nessus(object):
             if "report" in response["reports"]:
                 self._reports = []
                 for result in response["reports"]["report"]:
-                    self._reports.append(
-                        Report(result["name"], result["readableName"], result["status"], result["timestamp"])
-                    )
+                    r = self.Report()
+                    r.id = result["name"]
+                    r.name = result["name"]
+                    r.readable_name = result["readableName"]
+                    r.status = result["status"]
+                    r.timestamp = result["timestamp"]
+                    self._reports.append(r)
             return True
         else:
             return False
@@ -443,9 +476,6 @@ class Nessus(object):
             'tag_id': scan.tag.id,
             'custom_targets': scan.custom_targets
         }
-
-        print params
-
         if scan.target_file_name is not None:
             params['target_file_name'] = scan.target_file_name
 
@@ -587,6 +617,175 @@ class Nessus(object):
         else:
             return False
 
+    def load_scan_vulnerabilities(self, scan):
+        """
+        Load vulnerabilities results from scan.
+        Params:
+            scan(Scan): scan that we will load.
+        Returns:
+            True if successful, False otherwise.
+        """
+        params = {
+            'id': scan.id
+        }
+        response = self._api_request("POST", "/result/details", params)
+        if response is not None:
+            scan._vulnerabilities = []
+            for vulnerability in response["vulnerabilities"]:
+                v = Vulnerability()
+                v.plugin_id = vulnerability["plugin_id"]
+                v.plugin_name = vulnerability["plugin_name"]
+                v.plugin_family = vulnerability["plugin_family"]
+                v.severity = vulnerability["severity"]
+                v.severity_index = vulnerability["severity_index"]
+                v.count = vulnerability["count"]
+                v.vuln_index = vulnerability["vuln_index"]
+                #scan.vulnerabilities.append(v)
+            return True
+        else:
+            return False
+
+    def load_scan_remediations(self, scan):
+        """
+        Load a scan remediations content.
+        :param scan:
+        :return:
+        """
+        params = {
+            'id': scan.id
+        }
+        response = self._api_request("POST", "/result/details", params)
+        if response is not None:
+            scan._remediations = []
+            if "remediations" in response["remediations"]:
+                for remediation in response["remediations"]["remediations"]:
+                    r = Remediation()
+                    r.text = remediation["remediation"]
+                    r.value = remediation["value"]
+                    r.vulns = remediation["vulns"]
+                    r.hosts = remediation["hosts"]
+                    scan.remediations.append(r)
+            return True
+        else:
+            return False
+
+    def load_scan_notes(self, scan):
+        """
+        Load a scan notes.
+        Params:
+            scan(Scan):
+        Returns:
+        """
+        params = {
+            'id': scan.id
+        }
+        response = self._api_request("POST", "/result/details", params)
+        if response is not None:
+            scan.notes = []
+            if "note" in response["notes"]:
+                for note in response["notes"]["note"]:
+                    n = Note()
+                    n.title = note["title"]
+                    n.message = note["message"]
+                    n.severity = note["severity"]
+                    scan.notes.append(n)
+            return True
+        else:
+            return
+
+    def load_scan_hosts(self, scan):
+        """
+        Load a scan notes.
+        Params:
+            scan(Scan):
+        Returns:
+        """
+        params = {
+            'id': scan.id
+        }
+        response = self._api_request("POST", "/result/details", params)
+        if response is not None:
+            scan.hosts = []
+            for host in response["hosts"]:
+                    h = self.Host()
+                    h.scan = scan
+                    h.host_index = host["host_index"]
+                    h.totalchecksconsidered = host["totalchecksconsidered"]
+                    h.numchecksconsidered = host["numchecksconsidered"]
+                    h.scanprogresstotal = host["scanprogresstotal"]
+                    h.scanprogresscurrent = host["scanprogresscurrent"]
+                    h.score = host["score"]
+                    h.progress = host["progress"]
+                    h.critical = host["critical"]
+                    h.high = host["high"]
+                    h.medium = host["medium"]
+                    h.low = host["low"]
+                    h.info = host["info"]
+                    h.severity = host["severity"]
+                    h.host_id = host["host_id"]
+                    h.hostname = host["hostname"]
+                    scan.hosts.append(h)
+            return True
+        else:
+            return False
+
+    def load_host_info(self, host):
+        """
+
+        :param host:
+        :return:
+        """
+        params = {
+            'id': host.scan.id,
+            'host_id': host.id
+        }
+        response = self._api_request("POST", "/result/details", params)
+        if response is not None:
+            host.vulnerabilities = []
+            for vulnerability in response["vulnerabilities"]:
+                    v = Vulnerability()
+                    v.plugin_id = vulnerability["plugin_id"]
+                    v.plugin_name = vulnerability["plugin_name"]
+                    v.plugin_family = vulnerability["plugin_family"]
+                    v.severity = vulnerability["severity"]
+                    v.severity_index = vulnerability["severity_index"]
+                    v.count = vulnerability["count"]
+                    v.vuln_index = vulnerability["vuln_index"]
+                    host.vulnerabilities.append(v)
+            return True
+        else:
+            return False
+
+    def load_host_vulnerabilities(self, host):
+        params = {
+            'id': host.scan.id,
+            'host_id': host.id
+        }
+        response = self._api_request("POST", "/result/details", params)
+        if response is not None:
+            host.vulnerabilities = []
+            for vulnerability in response["vulnerabilities"]:
+                    v = Vulnerability()
+                    v.plugin_id = vulnerability["plugin_id"]
+                    v.plugin_name = vulnerability["plugin_name"]
+                    v.plugin_family = vulnerability["plugin_family"]
+                    v.severity = vulnerability["severity"]
+                    v.severity_index = vulnerability["severity_index"]
+                    v.count = vulnerability["count"]
+                    v.vuln_index = vulnerability["vuln_index"]
+                    host.vulnerabilities.append(v)
+            return True
+        else:
+            return False
+
+    def load_host_compliance(self, host):
+        """
+
+        :param host:
+        :return:
+        """
+        raise Exception("Not yet implemented.")
+
     def upload_file(self, filename):
         """
         Upload the file identified by filename to the server.
@@ -654,7 +853,6 @@ class Nessus(object):
         if policy.settings is not None:
             for k in policy.settings:
                 params[k] = policy.settings[k]
-        print params
         response = self._api_request("POST", "/policy/update", params)
         if response is not None:
             policy.id = response["metadata"]["id"]
@@ -664,16 +862,16 @@ class Nessus(object):
             policy.visibility = response["metadata"]["visibility"]
             self.load_policies()
             for _p in self.policies:
-                if _p.id == policy.id:
+                if int(_p.id) == int(policy.id):
                     policy.db_id = _p.db_id
                     policy.object_id = _p.object_id
             return True
         else:
             return False
 
-    def copy_policy(self, policy):
+    def clone_policy(self, policy):
         """
-        Copy a policy.
+        Clone a policy.
         Params:
             policy(Policy): policy instance to be created.
         Returns:
@@ -683,7 +881,7 @@ class Nessus(object):
         response = self._api_request("POST", "/policy/copy", params)
         if response is not None:
             _p = response["policy"]
-            p = Policy()
+            p = self.Policy()
             p.name = _p["policyname"]
             if "policycomments" in _p["policycontents"]:
                 p.description = _p["policycontents"]["policycomments"]
@@ -694,7 +892,7 @@ class Nessus(object):
             p.visibility = _p["visibility"]
             self.load_policies()
             for _p in self.policies:
-                if _p.id == p.id:
+                if int(_p.id) == int(p.id):
                     p.db_id = _p.db_id
                     p.object_id = _p.object_id
             return p
@@ -718,7 +916,25 @@ class Nessus(object):
         else:
             return False
 
-    def get_policy_preferences(self, policy):
+    def download_policy(self, policy, filename=None):
+        """
+        Download a policy description file.
+        Params:
+            policy(Policy): policy to be downloaded
+        Returns:
+            string: path where the policy has been saved.
+        """
+        if filename is None:
+            filename = "nessus_policy_%s.nessus" % policy.name
+        try:
+            response = self._request("GET", "/policy/download?policy_id=%d" % policy.id, "")
+            with open(filename, "wb") as f:
+                f.write(response)
+            return filename
+        except HTTPException as e:
+            return None
+
+    def load_policy_preferences(self, policy):
         """
         Load and assign policy preferences
         Params:
@@ -730,6 +946,7 @@ class Nessus(object):
         }
         response = self._api_request("POST", "/policy/list/plugins/preferences", params)
         if response is not None:
+            policy.preferences = []
             if "preference" in response["pluginpreferences"]:
                 for preference in response["pluginpreferences"]["preference"]:
                     p = Preference()
@@ -745,7 +962,7 @@ class Nessus(object):
         else:
             return False
 
-    def get_policy_plugins(self, policy):
+    def load_policy_plugins(self, policy):
         """
 
         :param policy:
@@ -1008,6 +1225,7 @@ class Nessus(object):
         params = {"id" : scan.uuid}
         response = self._api_request("POST", "/result/details", params)
         current = 0.0
+        scan.status = response["info"]["status"]
         total = 0.0 if len(response["hosts"]) else 1.0
         for host in response["hosts"]:
             current += host["scanprogresscurrent"]
